import type { Metadata } from 'next'
import Image from 'next/image'
import Link from 'next/link'
import { notFound } from 'next/navigation'
import { Suspense } from 'react'

import { buttonVariants } from '@/components/ui/button'
import { cn, sliceAndGetSlug } from '@/lib/utils'
import { getBlog } from '@/server/getBlog'
import axiosServer from '@/services/axiosServer'
import { SeoMeta } from '@/types/store.type'

import RecentArticles from './RecentArticles'

export const dynamic = 'force-dynamic'

export async function generateMetadata(): Promise<Metadata> {
  const res = await axiosServer('api/get-static-metas/blog')
  const seo: SeoMeta = res.data.data

  if (!seo) return notFound()
  return {
    title: seo.meta_title,
    description: seo.meta_description,
    keywords: seo.meta_keywords,
    publisher: 'SiaMoods',
    category: 'website',
    openGraph: {
      images: '/images/logo.png',
      title: seo.meta_title,
      description: seo.meta_description,
      type: 'website',
      siteName: 'SiaMoods',
      url: `${process.env.NEXT_PUBLIC_APP_URL}blog`,
    },
    alternates: {
      canonical: `${process.env.NEXT_PUBLIC_APP_URL}blog`,
    },
  }
}

export default async function page() {
  const blogData = await getBlog()

  return (
    <div className='container flex gap-12 pb-8 lg:flex-col-reverse'>
      <Suspense fallback={<div>Loading...</div>}>
        <RecentArticles />
      </Suspense>
      <div>
        <h1 className='text-2xl font-bold'>Բլոգ</h1>
        <div className='divide-y'>
          {blogData?.blog.map((item, index) => (
            <article
              key={index}
              className='flex flex-col items-start gap-2 py-10'
            >
              <h4 className='text-xl font-bold sm:text-lg'>{item.title}</h4>
              <div className='text-sm text-muted-foreground'>{item.date}</div>
              <Link
                href={`blog/${sliceAndGetSlug(item.slug)}`}
                className='w-full'
              >
                <Image
                  src={item.image}
                  alt={item.title}
                  width={900}
                  height={500}
                  className='my-4 aspect-[21/9] w-full rounded-md object-cover'
                />
              </Link>
              <Link
                href={`/blog/${item.slug}`}
                className={cn(buttonVariants())}
              >
                Կարդալ ավելին
              </Link>
            </article>
          ))}
        </div>
      </div>
    </div>
  )
}
