'use client'

import Link from 'next/link'

import { useMutation, useQuery, useQueryClient } from '@tanstack/react-query'
import { AxiosError } from 'axios'

import Spinner from '@/components/ui/Spinner'
import { Button, buttonVariants } from '@/components/ui/button'
import {
  Table,
  TableBody,
  TableCell,
  TableFooter,
  TableHead,
  TableHeader,
  TableRow,
} from '@/components/ui/table'
import { useToast } from '@/components/ui/use-toast'
import { cn, convertToMoney } from '@/lib/utils'
import { getCart } from '@/server/getCart'
import axiosClient from '@/services/axiosClient'

import CartItem from './CartItem'

const Cart = () => {
  // const totalPrice = cartDataMock.reduce(
  //   (acc, item) => acc + item.price * item.quantity,
  //   0
  // )
  const { data: cartData, isPending } = useQuery({
    queryKey: ['cart'],
    queryFn: getCart,
  })

  const { toast } = useToast()
  const queryClient = useQueryClient()

  const deleteFromCartMutation = useMutation({
    mutationFn: (id: number) => axiosClient.delete(`api/carts/${id}`),
    onSuccess: () => {
      queryClient.invalidateQueries({ queryKey: ['cart'] })
    },
    onError: (error) => {
      if (error instanceof AxiosError) {
        if (error?.response?.data.message) {
          toast({
            title: error.response.data.message,
            variant: 'destructive',
          })
        }
      }
    },
  })

  return (
    <div className='container py-4 pt-0'>
      <h1 className='pb-4 text-2xl font-bold sm:text-xl'>
        Զամբյուղի պարունակություն
      </h1>
      {isPending ? (
        <Spinner isSpinning={isPending} />
      ) : (
        <>
          {!cartData || cartData?.products?.length === 0 ? (
            <div className='mt-4 space-y-4 rounded-lg bg-muted p-12 text-center text-lg text-muted-foreground'>
              <h2>Ձեր զամբյուղը դատարկ է</h2>
              <Link href='/store' className={cn(buttonVariants())}>
                Շարունակել Գնումները
              </Link>
            </div>
          ) : (
            <div className='flex items-start gap-8 lg:flex-col'>
              <div className='flex-[2] lg:w-full'>
                <Table className='sm:hidden'>
                  <TableHeader>
                    <TableRow className='hover:bg-background'>
                      <TableHead className='font-bold'>ԱՊՐԱՆՔ</TableHead>
                      <TableHead className='w-[80px]  text-end font-bold'>
                        ԳԻՆ
                      </TableHead>
                      <TableHead className='w-[100px] text-center font-bold'>
                        ՔԱՆԱԿ
                      </TableHead>
                      <TableHead className='w-[100px] text-end font-bold'>
                        ԸՆԴԱՄԵՆԸ
                      </TableHead>
                      <TableHead className='w-[20px] pl-0 text-end font-bold'></TableHead>
                    </TableRow>
                  </TableHeader>
                  <TableBody>
                    {cartData?.products?.map((item) => (
                      <CartItem
                        key={item.id}
                        item={item.product}
                        itemId={item.id}
                        itemVariation={item.variation}
                        itemQuantity={item.quantity}
                        deleteFromCartMutation={deleteFromCartMutation}
                        isPending={deleteFromCartMutation.isPending}
                        isTableLayout={true}
                      />
                    ))}
                  </TableBody>
                  <TableFooter>
                    <TableRow>
                      <TableCell colSpan={5}>
                        <Link
                          href='/store'
                          className={cn(
                            buttonVariants({
                              variant: 'outline',
                              className: 'font-bold',
                            })
                          )}
                        >
                          Շարունակել Գնումները
                        </Link>
                      </TableCell>
                    </TableRow>
                  </TableFooter>
                </Table>
                <div className='hidden flex-col items-center gap-4 sm:flex'>
                  {cartData?.products?.map((item) => (
                    <CartItem
                      key={item.id}
                      item={item.product}
                      itemId={item.id}
                      itemVariation={item.variation}
                      itemQuantity={item.quantity}
                      deleteFromCartMutation={deleteFromCartMutation}
                      isPending={deleteFromCartMutation.isPending}
                      isTableLayout={false}
                    />
                  ))}
                </div>
              </div>
              <div className='sticky top-28 flex flex-1 flex-col gap-8 rounded-md bg-muted p-8 lg:w-full'>
                <h2 className='border-b pb-2 font-bold'>ՊԱՏՎԵՐԻ ԱՐԺԵՔԸ</h2>
                <div className='flex justify-between gap-2'>
                  <div>Ընդհանուր</div>
                  <div className='font-bold'>
                    {convertToMoney(cartData?.sum)}
                  </div>
                </div>
                <Link href='/checkout'>
                  <Button className='w-full'>ԱՆՑՆԵԼ ՎՃԱՐՄԱՆԸ</Button>
                </Link>
              </div>
            </div>
          )}
        </>
      )}
    </div>
  )
}

export default Cart
