import Link from 'next/link'
import { useEffect, useState } from 'react'

import {
  UseMutationResult,
  useMutation,
  useQueryClient,
} from '@tanstack/react-query'
import { AxiosResponse } from 'axios'
import { Trash2 } from 'lucide-react'
import { useDebounce } from 'use-debounce'

import ImageWithFallback from '@/components/ImageWithFallback'
import { Button } from '@/components/ui/button'
import { InputQuantity } from '@/components/ui/inputQuantity'
import { TableCell, TableRow } from '@/components/ui/table'
import { convertToMoney, formatInputValue } from '@/lib/utils'
import axiosClient from '@/services/axiosClient'
import type { ProductType, Variation } from '@/types'

export default function CartItem({
  item,
  itemId,
  itemQuantity,
  itemVariation,
  isPending,
  deleteFromCartMutation,
  isTableLayout = true,
}: {
  item: ProductType
  itemId: number
  itemQuantity: number
  isPending: boolean
  itemVariation: Variation | null
  deleteFromCartMutation: UseMutationResult<
    AxiosResponse<any, any>,
    Error,
    number,
    unknown
  >
  isTableLayout?: boolean // Flag for layout type
}) {
  const [quantity, setQuantity] = useState<number | string>(itemQuantity || 1)
  const [isQuantityChanged, setIsQuantityChanged] = useState(false)
  const [debouncedQuantity] = useDebounce(quantity, 500)
  const queryClient = useQueryClient()

  const changeCartItemQuantityMutation = useMutation({
    mutationFn: () =>
      axiosClient.post(`api/carts`, {
        products: [
          {
            haysell_id: item.haysell_id,
            quantity: debouncedQuantity,
            variation_id: itemVariation?.id,
          },
        ],
        is_cart: true,
      }),
    onSuccess: () => {
      queryClient.invalidateQueries({ queryKey: ['cart'] })
    },
    onError: (error) => {
      console.log(error)
    },
  })

  useEffect(() => {
    if (isQuantityChanged) {
      changeCartItemQuantityMutation.mutate()
    }
  }, [debouncedQuantity])

  if (isTableLayout) {
    return (
      <TableRow className='hover:bg-background'>
        <TableCell className='font-medium'>
          <div className='flex items-center gap-4'>
            <Link href={`/product/${item.slug}`}>
              <ImageWithFallback
                src={item.images.length > 0 ? item.images[0].image : ''}
                alt={item.title}
                width={40}
                height={40}
                className='rounded-sm'
              />
            </Link>
            <Link
              href={`/product/${item.slug}`}
              className='transition-colors hover:text-primary'
            >
              <div>{item.title}</div>
              {itemVariation?.title && <div>Չափս։ {itemVariation?.title}</div>}
            </Link>
          </div>
        </TableCell>
        <TableCell className='whitespace-nowrap font-bold'>
          {convertToMoney(item.price)}
        </TableCell>
        <TableCell>
          <InputQuantity
            value={quantity}
            setValue={(value) => {
              setQuantity(value)
              setIsQuantityChanged(true)
            }}
            onChange={(e) => {
              const value = formatInputValue(
                e.target.value,
                'integer'
              ) as string
              setIsQuantityChanged(true)
              if (+value < 1) {
                setQuantity(1)
                return
              }
              if (item.available == 0) {
                setQuantity(itemQuantity)
                return
              }
              if (item.available < +value) {
                setQuantity(item.available)
                return
              }
              setQuantity(value)
            }}
            available={item.available}
          />
        </TableCell>
        <TableCell className='whitespace-nowrap text-right font-bold'>
          {convertToMoney(item.price * itemQuantity)}
        </TableCell>
        <TableCell className='pl-0'>
          <Button
            variant='ghost'
            size='iconSm'
            className='text-muted-foreground'
            disabled={isPending}
            onClick={() => deleteFromCartMutation.mutate(itemId)}
          >
            <Trash2 size={20} />
          </Button>
        </TableCell>
      </TableRow>
    )
  }

  return (
    <div className='relative flex w-full flex-col items-center gap-2 rounded-md p-4 shadow-md'>
      <Button
        variant='ghost'
        size='iconSm'
        className='absolute right-0 top-0 text-muted-foreground'
        disabled={isPending}
        onClick={() => deleteFromCartMutation.mutate(itemId)}
      >
        <Trash2 size={20} />
      </Button>
      <div className='flex items-center gap-8'>
        <Link href={`/product/${item.slug}`}>
          <ImageWithFallback
            src={item.images.length > 0 ? item.images[0].image : ''}
            alt={item.title}
            width={80}
            height={80}
            className='rounded-sm'
          />
        </Link>
        <InputQuantity
          value={quantity}
          setValue={(value) => {
            setQuantity(value)
            setIsQuantityChanged(true)
          }}
          onChange={(e) => {
            const value = formatInputValue(e.target.value, 'integer') as string
            setIsQuantityChanged(true)
            if (+value < 1) {
              setQuantity(1)
              return
            }
            if (item.available == 0) {
              setQuantity(itemQuantity)
              return
            }
            if (item.available < +value) {
              setQuantity(item.available)
              return
            }
            setQuantity(value)
          }}
          available={item.available}
        />
      </div>
      <div className='flex w-full flex-col items-center gap-2'>
        <Link
          href={`/product/${item.slug}`}
          className='transition-colors hover:text-primary'
        >
          <div>{item.title}</div>
          {/* {item.variant && (
            <div className='text-center'>Չափս։ {item.variant}</div>
          )} */}
        </Link>
        <div className='flex w-full justify-between gap-2'>
          <div>
            Գին:{' '}
            <span className='whitespace-nowrap font-bold'>
              {convertToMoney(item.price)}
            </span>
          </div>
          <div>
            Ընդամենը:{' '}
            <span className='whitespace-nowrap font-bold'>
              {convertToMoney(item.price * itemQuantity)}
            </span>
          </div>
        </div>
      </div>
    </div>
  )
}
