import type { Metadata } from 'next'
import { Inter as FontSans } from 'next/font/google'
import { notFound } from 'next/navigation'

import { GoogleOAuthProvider } from '@react-oauth/google'
import {
  HydrationBoundary,
  QueryClient,
  dehydrate,
} from '@tanstack/react-query'

import FacebookInit from '@/components/FacebookInit'
import Footer from '@/components/Footer'
import GenerateToken from '@/components/GenerateToken'
import LoadScripts from '@/components/LoadScripts'
import ProgressbarProvider from '@/components/ProgressbarProvider'
import ReactQueryClientProvider from '@/components/ReactQueryClientProvider'
import Header from '@/components/header/Header'
import { Toaster } from '@/components/ui/toaster'
import { cn } from '@/lib/utils'
import axiosServer from '@/services/axiosServer'
import { HomeData } from '@/types'
import { SeoMeta } from '@/types/store.type'

import './globals.css'
import './reset.css'

const fontSans = FontSans({
  subsets: ['latin'],
  variable: '--font-sans',
})

export async function generateMetadata(): Promise<Metadata> {
  const res = await axiosServer('api/get-static-metas/home')
  const seo: SeoMeta = res.data.data

  if (!seo) return notFound()
  return {
    title: seo.meta_title,
    description: seo.meta_description,
    keywords: seo.meta_keywords,
    publisher: 'SiaMoods',
    category: 'website',
    openGraph: {
      images: '/images/logo.png',
      title: seo.meta_title,
      description: seo.meta_description,
      type: 'website',
      siteName: 'SiaMoods',
      url: `${process.env.NEXT_PUBLIC_APP_URL}`,
    },
    alternates: {
      canonical: `${process.env.NEXT_PUBLIC_APP_URL}`,
    },
  }
}

export default async function RootLayout({
  children,
}: Readonly<{
  children: React.ReactNode
}>) {
  const queryClient = new QueryClient()
  await queryClient.prefetchQuery({
    queryKey: ['home'],
    queryFn: async (): Promise<HomeData | undefined> => {
      const res = await axiosServer('api/get-home')
      return res.data.data
    },
  })

  return (
    <html lang='en'>
      <body
        className={cn(
          'min-h-screen bg-background font-sans antialiased',
          fontSans.variable
        )}
      >
        <ReactQueryClientProvider>
          <HydrationBoundary state={dehydrate(queryClient)}>
            <ProgressbarProvider>
              <LoadScripts />

              <GoogleOAuthProvider
                clientId={process.env.NEXT_PUBLIC_GOOGLE_CLIENT_ID!}
              >
                <FacebookInit />
                <GenerateToken />
                <Header />
                {children}
                <Footer />
                <Toaster />
              </GoogleOAuthProvider>
            </ProgressbarProvider>
          </HydrationBoundary>
        </ReactQueryClientProvider>
      </body>
    </html>
  )
}
