'use client'

import Link from 'next/link'
import { useRef, useState } from 'react'

import { zodResolver } from '@hookform/resolvers/zod'
import { useQuery } from '@tanstack/react-query'
import axios, { AxiosError } from 'axios'
import { motion } from 'framer-motion'
import Cookies from 'js-cookie'
import ReCAPTCHA from 'react-google-recaptcha'
import { useForm } from 'react-hook-form'
import * as z from 'zod'

import SocialLogin from '@/components/SocialLogin'
import { Button, buttonVariants } from '@/components/ui/button'
import {
  Form,
  FormControl,
  FormField,
  FormItem,
  FormLabel,
  FormMessage,
} from '@/components/ui/form'
import { Input } from '@/components/ui/input'
import { useToast } from '@/components/ui/use-toast'
import { cn } from '@/lib/utils'
import { getCart } from '@/server/getCart'
import axiosClient from '@/services/axiosClient'
import { setNextCookies } from '@/utils/setNextCookies'

const formSchema = z.object({
  email: z.string().min(1, { message: 'Էլ․ հասցեն պարտադիր է' }).email({
    message: 'Խնդրում ենք մուտքագրել վավեր էլ․ հասցե',
  }),
  password: z.string().min(1, { message: 'Գաղտնաբառը պարտադիր է' }),
})

const Login = () => {
  const { toast } = useToast()
  const [loading, setLoading] = useState(false)
  const recaptchaRef = useRef<ReCAPTCHA>(null)
  const [captchaToken, setCaptchaToken] = useState<string | null>(null)
  const { data: cartData } = useQuery({
    queryKey: ['cart'],
    queryFn: getCart,
  })
  const form = useForm({
    resolver: zodResolver(formSchema),
    defaultValues: {
      email: '',
      password: '',
    },
  })

  async function handleLogin(values: z.infer<typeof formSchema>) {
    setLoading(true)
    try {
      await axiosClient('sanctum/csrf-cookie')
      const res = await axiosClient.post('api/account-login', {
        ...values,
        captchaToken,
      })
      const token = res.data.token
      if (cartData && cartData?.products?.length > 0) {
        await axios.post(
          `${process.env.NEXT_PUBLIC_API_URL}api/carts`,
          {
            products: cartData.products?.map((item) => ({
              haysell_id: item.product.haysell_id,
              quantity: item.quantity,
              variation_id: item.variation?.id,
            })),
            is_cart: true,
          },
          {
            headers: {
              Authorization: `Bearer ${token}`,
            },
          }
        )
      }

      Cookies.set('siamoods_token', token)
      setNextCookies('siamoods_token', token)
      window.location.href = '/'
    } catch (error) {
      recaptchaRef.current?.reset()
      if (error instanceof AxiosError) {
        if (error?.response?.data.message) {
          toast({
            title: error.response.data.message,
            variant: 'destructive',
          })
        }
      }
    }
    setLoading(false)
  }

  return (
    <motion.div
      initial={{ opacity: 0 }}
      animate={{ opacity: 1 }}
      exit={{ opacity: 0 }}
      transition={{
        type: 'spring',
        stiffness: 260,
        damping: 40,
      }}
      className='flex justify-center'
    >
      <div className='m-10 flex h-[680px] max-w-5xl overflow-hidden rounded-2xl shadow-2xl lg:m-4 md:h-auto md:flex-col'>
        <Form {...form}>
          <form
            onSubmit={form.handleSubmit(handleLogin)}
            className='flex flex-[1.2] flex-col items-center justify-center gap-6 p-12 pt-6 lg:p-4'
          >
            <div className='flex flex-col items-center gap-4'>
              <h1>Մուտք</h1>
              <p className='text-muted-foreground'>
                Խնդրում ենք լրացնել դաշտերը
              </p>
            </div>
            <FormField
              name='email'
              render={({ field }) => (
                <FormItem>
                  <FormLabel required={true}>էլ․ հասցե</FormLabel>
                  <FormControl>
                    <Input {...field} type='email' />
                  </FormControl>
                  <FormMessage />
                </FormItem>
              )}
            />
            <div className='flex w-full flex-col items-end gap-2 sm:items-center sm:gap-6'>
              <FormField
                name='password'
                render={({ field }) => (
                  <FormItem>
                    <FormLabel required={true}>Գաղտնաբառ</FormLabel>
                    <FormControl>
                      <Input {...field} type='password' />
                    </FormControl>
                    <FormMessage />
                  </FormItem>
                )}
              />
              <Link
                href='/forgot-password'
                className='text-xs text-primary hover:underline'
              >
                Մոռացել եք գաղտնաբառը?
              </Link>
            </div>
            <ReCAPTCHA
              ref={recaptchaRef}
              sitekey={process.env.NEXT_PUBLIC_RE_CAPTCHA_KEY as string}
              onChange={(token) => setCaptchaToken(token)}
            />
            <Button className='w-full font-bold' disabled={loading}>
              Մուտք
            </Button>
            <SocialLogin />
          </form>
        </Form>

        <div className='relative flex flex-1 flex-col items-center justify-center gap-8 p-8'>
          <div className='gradient' />
          <p className='text-center text-muted-foreground'>
            Ստեղծեք հաշիվ և օգտագործեք այն՝ պատվերը ձևակերպելու համար
          </p>
          <Link
            href='/register'
            className={cn(
              buttonVariants({
                variant: 'outlineSecondary',
              })
            )}
          >
            Ստեղծել հաշիվ
          </Link>
        </div>
      </div>
    </motion.div>
  )
}

export default Login
