'use client'

import Link from 'next/link'
import { useState } from 'react'

import { useMutation, useQuery, useQueryClient } from '@tanstack/react-query'
import { AxiosError } from 'axios'
import { X } from 'lucide-react'
import { IoBagHandleOutline } from 'react-icons/io5'

import {
  Tooltip,
  TooltipContent,
  TooltipProvider,
  TooltipTrigger,
} from '@/components/ui/tooltip'
import { cn, convertToMoney } from '@/lib/utils'
import { getCart } from '@/server/getCart'
import axiosClient from '@/services/axiosClient'

import ImageWithFallback from './ImageWithFallback'
import { Button, buttonVariants } from './ui/button'
import { useToast } from './ui/use-toast'

const CartPopup = () => {
  const [open, setOpen] = useState(false)

  const { toast } = useToast()
  const queryClient = useQueryClient()
  const { data: cartData } = useQuery({
    queryKey: ['cart'],
    queryFn: getCart,
  })
  const itemsCount = cartData?.products?.reduce(
    (acc, item) => acc + item.quantity,
    0
  )

  // console.log('🚀 ~ file: CartPopup.tsx:33 ~ cartData:', cartData)

  const deleteFromCartPopupMutation = useMutation({
    mutationFn: (id: number) => axiosClient.delete(`api/carts/${id}`),
    onSuccess: () => {
      queryClient.invalidateQueries({ queryKey: ['cart'] })
    },
    onError: (error) => {
      if (error instanceof AxiosError) {
        if (error?.response?.data.message) {
          toast({
            title: error.response.data.message,
            variant: 'destructive',
          })
        }
      }
    },
  })

  return (
    <TooltipProvider delayDuration={0}>
      <Tooltip open={open} onOpenChange={setOpen}>
        <TooltipTrigger asChild>
          <Link
            href='/cart'
            className={cn(
              buttonVariants({
                variant: 'ghost',
                size: 'iconSm',
                className: 'sm:size-10',
              })
            )}
          >
            <div className='relative'>
              {!!itemsCount && (
                <div className='absolute -end-1 -top-1 flex size-[0.875rem] items-center justify-center rounded-full bg-primary text-[0.625rem] text-background'>
                  <div>{itemsCount}</div>
                </div>
              )}
              <i className='text-2xl'>
                <IoBagHandleOutline className='[&>path]:stroke-[40]' />
              </i>
            </div>
          </Link>
        </TooltipTrigger>
        <TooltipContent className='p-0 md:hidden' align='end'>
          <div className='flex w-[25rem] flex-col'>
            {cartData?.products && cartData.products.length > 0 && (
              <div className='custom-scrollbar max-h-[26rem] w-full divide-y overflow-y-auto px-2 py-2'>
                {cartData.products.map((item) => (
                  <div
                    key={item.id}
                    className='relative flex items-center gap-4 py-2 text-muted-foreground'
                  >
                    <Link href={`/product/${item.product?.slug}`}>
                      <ImageWithFallback
                        src={
                          (item.product.images.length > 0 &&
                            item.product.images[0].image) ||
                          ''
                        }
                        alt='product'
                        width={80}
                        height={80}
                        className='aspect-square rounded-md object-cover'
                      />
                    </Link>
                    <div className='flex flex-1 flex-col gap-1 pe-5'>
                      <Link
                        href={`/product/${item.product?.slug}`}
                        className='line-clamp-1 text-base transition-colors hover:text-primary'
                      >
                        {item.product?.title}
                      </Link>
                      {item.variation?.title && (
                        <div>Չափս: {item.variation?.title}</div>
                      )}
                      <div>
                        {item.quantity} x {convertToMoney(item.product?.price)}
                      </div>
                    </div>
                    <Button
                      variant='ghost'
                      size='iconSm'
                      className='absolute -right-2 -top-0 rounded-full p-2'
                      disabled={deleteFromCartPopupMutation.isPending}
                      onClick={() =>
                        deleteFromCartPopupMutation.mutate(item.id)
                      }
                    >
                      <X size={18} />
                    </Button>
                  </div>
                ))}
              </div>
            )}
            {(!cartData || cartData?.products?.length === 0) && (
              <div className='p-8 text-center text-muted-foreground'>
                Զամբյուղը դատարկ է
              </div>
            )}
            {cartData?.products && cartData.products.length > 0 && (
              <div className='flex flex-col gap-4 border-t p-4 text-base'>
                <div className='flex justify-between gap-2 text-muted-foreground'>
                  <div>Ընդհանուր:</div>
                  <div className='font-bold'>
                    {convertToMoney(cartData?.sum)}
                  </div>
                </div>
                <div className='flex gap-4'>
                  <Link
                    href='/cart'
                    className={cn(
                      buttonVariants({
                        variant: 'outline',
                        className: 'flex-1',
                      })
                    )}
                    onClick={() => setOpen(false)}
                  >
                    Զամբյուղ
                  </Link>
                  <Link
                    href='/checkout'
                    className={cn(
                      buttonVariants({
                        className: 'flex-1',
                      })
                    )}
                    onClick={() => setOpen(false)}
                  >
                    Պատվիրել
                  </Link>
                </div>
              </div>
            )}
          </div>
        </TooltipContent>
      </Tooltip>
    </TooltipProvider>
  )
}

export default CartPopup
