'use client'

import Link from 'next/link'
import { useEffect, useState } from 'react'

import { useQuery, useQueryClient } from '@tanstack/react-query'
import Cookies from 'js-cookie'
import { ChevronLeft, ChevronRight } from 'lucide-react'

import AddToCart from '@/components/AddToCart'
import Magnifier from '@/components/Magnifier'
import { Button } from '@/components/ui/button'
import {
  Dialog,
  DialogContent,
  DialogHeader,
  DialogTrigger,
} from '@/components/ui/dialog'
import {
  Select,
  SelectContent,
  SelectGroup,
  SelectItem,
  SelectTrigger,
  SelectValue,
} from '@/components/ui/select'
import {
  cn,
  convertToMoney,
  encodeImageUrl,
  formatInputValue,
} from '@/lib/utils'
import { getProduct } from '@/server/getProduct'
import { useUserStore } from '@/store/user'
import { Variation } from '@/types'

import CardCarousel from './CardCarousel'
import ImageWithFallback from './ImageWithFallback'
import SetBreadcrumbItem from './SetBreadcrumbItem'
import SocialShare from './SocialShare'
import AddReviewDialog from './dialogs/AddReviewDialog'
import Spinner from './ui/Spinner'
import { StarRatings } from './ui/starRating'

const Product = ({
  isInDialog,
  className,
  slug,
}: {
  isInDialog?: boolean
  className?: string
  slug: string
}) => {
  const queryClient = useQueryClient()
  const [quantity, setQuantity] = useState<number | string>(1)
  const [isGalleryOpen, setIsGalleryOpen] = useState(false)
  const recentlyViewedProductsIds: number[] = JSON.parse(
    Cookies.get('recentlyViewedProductsIds') || '[]'
  )
  const addFavorite = useUserStore((state) => state.addFavorite)
  const {
    data: productData,
    isPending,
    isFetched,
    isSuccess,
  } = useQuery({
    queryKey: ['product', slug],
    queryFn: () => getProduct(slug),
    staleTime: 1000 * 60 * 60,
    enabled: !!slug,
  })

  useEffect(() => {
    if (isSuccess && process.env.NODE_ENV === 'production') {
      fbq('track', 'ViewContent', {
        content_ids: productData?.haysell_id,
        content_type: 'product',
      })
    }
  }, [isSuccess])

  const haySellId = productData?.haysell_id
  const firstVariant =
    (productData?.variations && productData.variations[0]) || null
  const hasVariants =
    productData?.variations && productData.variations.length > 0 ? true : false
  const [currentVariant, setCurrentVariant] = useState<Variation | null>(
    firstVariant
  )
  const price = hasVariants ? currentVariant?.price : productData?.price
  const discountPrice = hasVariants
    ? currentVariant?.discount_price
    : productData?.discount_price
  const discount = hasVariants
    ? currentVariant?.discount
    : productData?.discount
  const available = hasVariants
    ? currentVariant?.available ?? 0
    : productData?.available ?? 0
  const [selectedImage, setSelectedImage] = useState(
    encodeImageUrl(
      (productData &&
        productData.images.length > 0 &&
        productData.images[0].image) ||
        ''
    ) ?? '/icons/diamond-ring.png'
  )

  useEffect(() => {
    if (productData) {
      setCurrentVariant(firstVariant)
    }
  }, [productData])

  useEffect(() => {
    if (haySellId) {
      if (productData?.is_favorite) {
        addFavorite(haySellId)
      }

      // If haysell_id exists in recentlyViewedProductsIds.
      // Move it to the start. If not, add it.
      // Keep max length 10, remove last if length exceeds.
      const index = recentlyViewedProductsIds.indexOf(haySellId)

      if (index !== -1) {
        recentlyViewedProductsIds.splice(index, 1)
      }

      recentlyViewedProductsIds.unshift(haySellId)

      if (recentlyViewedProductsIds.length > 10) {
        recentlyViewedProductsIds.pop()
      }

      Cookies.set(
        'recentlyViewedProductsIds',
        JSON.stringify(recentlyViewedProductsIds)
      )
      queryClient.invalidateQueries({
        queryKey: ['recentlyViewedProductsIds'],
      })
    }
  }, [])

  useEffect(() => {
    if (productData?.images && productData.images.length > 0) {
      setSelectedImage(encodeImageUrl(productData.images[0].image))
    }
  }, [isFetched])

  useEffect(() => {
    if (isGalleryOpen && productData?.images && productData.images.length > 1) {
      const handleKeyDown = (event: KeyboardEvent) => {
        if (event.key === 'ArrowRight') {
          setSelectedImage(encodeImageUrl(productData.images[1].image))
        } else if (event.key === 'ArrowLeft') {
          setSelectedImage(encodeImageUrl(productData.images[0].image))
        }
      }

      window.addEventListener('keydown', handleKeyDown)

      return () => {
        window.removeEventListener('keydown', handleKeyDown)
      }
    }
  }, [isGalleryOpen, productData])

  return (
    <div className='flex flex-col gap-8'>
      <SetBreadcrumbItem slug={slug} title={productData?.title} />
      <Spinner isSpinning={isPending} />
      <div className={cn('flex gap-8  lg:flex-col lg:items-center', className)}>
        <div className='flex flex-col gap-8'>
          <Dialog open={isGalleryOpen} onOpenChange={setIsGalleryOpen}>
            <DialogTrigger>
              <Magnifier src={selectedImage} isInDialog={isInDialog} />
            </DialogTrigger>
            <DialogContent
              className='max-w-fit gap-0 p-0'
              onOpenAutoFocus={(e) => e.preventDefault()}
            >
              <DialogHeader className='bg-muted p-4 text-start text-muted-foreground'>
                <div>{productData?.title}</div>
              </DialogHeader>
              <div className='relative aspect-square h-[calc(100dvh-6rem)] sm:h-auto sm:w-screen'>
                <ImageWithFallback
                  src={selectedImage}
                  alt=''
                  className='cursor-pointer rounded-md border'
                  width={1080}
                  height={1080}
                />

                {productData?.images && productData.images.length > 1 && (
                  <div>
                    <Button
                      onClick={() => {
                        setSelectedImage(
                          encodeImageUrl(productData.images[0].image)
                        )
                      }}
                      variant='ghost'
                      className='absolute -left-2 top-1/2 h-24 -translate-y-1/2 hover:bg-transparent [&>div]:hover:bg-primary [&>div]:hover:text-background'
                      disabled={selectedImage === productData.images[0].image}
                    >
                      <div className='rounded-full bg-background p-2 text-primary transition-colors'>
                        <ChevronLeft size={40} />
                      </div>
                    </Button>
                    <Button
                      onClick={() => {
                        setSelectedImage(
                          encodeImageUrl(productData.images[1].image)
                        )
                      }}
                      variant='ghost'
                      className='absolute -right-2 top-1/2 h-24 -translate-y-1/2 hover:bg-transparent [&>div]:hover:bg-primary [&>div]:hover:text-background'
                      disabled={selectedImage === productData.images[1].image}
                    >
                      <div className='rounded-full bg-background p-2 text-primary transition-colors'>
                        <ChevronRight size={40} />
                      </div>
                    </Button>
                  </div>
                )}
              </div>
            </DialogContent>
          </Dialog>

          <div className='flex gap-2'>
            {productData?.images?.map((image, index) => (
              <ImageWithFallback
                key={index}
                src={image.image}
                alt=''
                // alt={productData.title}
                onClick={() => setSelectedImage(encodeImageUrl(image.image))}
                className={cn(
                  'cursor-pointer rounded-md border',
                  selectedImage === image.image && 'border-primary'
                )}
                width={100}
                height={100}
              />
            ))}
          </div>
        </div>
        <div className='flex max-w-[600px] flex-col items-start gap-4 lg:items-center'>
          <h1 className='text-center text-2xl font-bold sm:text-xl'>
            {productData?.title}
          </h1>
          {!!discount && (
            <div className='relative text-sm'>
              {convertToMoney(price)}
              <span className='absolute left-0 top-1/2 h-px w-full bg-red-500'></span>
            </div>
          )}
          <div className='text-lg font-bold text-primary'>
            {discountPrice
              ? convertToMoney(discountPrice)
              : convertToMoney(price)}
          </div>
          {price && discountPrice && (
            <div className='text-sm'>
              Դուք խնայում եք {convertToMoney(price - discountPrice)} (
              {discount}%)
            </div>
          )}
          {productData?.variations && productData?.variations?.length > 0 && (
            <div className='flex items-center gap-12'>
              <div>{productData.variation_type}</div>
              <Select
                onValueChange={(value) => {
                  const variant = productData.variations.find(
                    (variant) => variant.id == value
                  )
                  if (variant) {
                    setCurrentVariant(variant)
                    setQuantity(1)
                  }
                }}
              >
                <SelectTrigger className='w-32'>
                  <SelectValue placeholder={productData.variations[0].title} />
                </SelectTrigger>
                <SelectContent>
                  <SelectGroup>
                    {productData.variations.map((variant) => (
                      <SelectItem key={variant.id} value={variant.id}>
                        {variant.title}
                      </SelectItem>
                    ))}
                  </SelectGroup>
                </SelectContent>
              </Select>
            </div>
          )}
          <div
            className={cn(
              'text-sm font-bold',
              available > 0 ? 'text-green-500' : 'text-destructive'
            )}
          >
            {available > 0 ? 'Առկա է' : 'Առկա չէ'}
          </div>
          <div className='text-start text-sm lg:text-center'>
            <p
              dangerouslySetInnerHTML={{
                __html: productData?.specification ?? '',
              }}
            ></p>
            {productData?.weight && <div>Քաշը - {productData.weight}</div>}
            {productData?.size && <div>Չափսեր - {productData.size}</div>}
          </div>
          <AddToCart
            product={productData}
            haysellId={haySellId}
            quantity={quantity}
            setQuantity={setQuantity}
            variant={currentVariant?.id}
            onChange={(e) => {
              const value = formatInputValue(
                e.target.value,
                'integer'
              ) as string
              if (+value < 1) {
                setQuantity(1)
                return
              }
              if (available < +value) {
                setQuantity(available)
                return
              }
              setQuantity(value)
            }}
            available={available}
            isFavoriteInitial={productData?.is_favorite}
          />
          {!isInDialog && <SocialShare className='pt-2' />}
          {!isInDialog &&
            productData?.linked_title &&
            productData?.linked_products &&
            productData.linked_products.length > 0 && (
              <Link
                href='/store/shghthaner'
                className='-mt-4 self-center text-xl'
              >
                {productData.linked_title}
              </Link>
            )}
          {!isInDialog &&
            productData?.linked_products &&
            productData.linked_products.length > 0 && (
              <CardCarousel
                showDots={false}
                containerClassName='pt-0 -mt-4'
                isInProduct={true}
                products={productData.linked_products}
                alwaysShowButtons={true}
              />
            )}
        </div>
      </div>
      {!isInDialog && productData?.description && (
        <p className='text-sm'>{productData.description}</p>
      )}

      {!isInDialog && (
        <>
          <div className='flex w-full flex-col items-center gap-4'>
            {productData?.reviews && productData.reviews.length > 0 ? (
              <div className='flex w-full flex-col gap-4 py-4'>
                {productData.reviews.map((review, index) => (
                  <div
                    key={index}
                    className='flex w-full flex-col gap-2 rounded-md border p-4'
                  >
                    <div className='flex justify-between'>
                      <div>{review.name}</div>
                      <div className='text-sm text-muted-foreground'>
                        {review.date}
                      </div>
                    </div>
                    <StarRatings
                      variant='primary'
                      rating={review.rating}
                      readOnly={true}
                    />
                    <div className='text-sm'>{review.review}</div>
                  </div>
                ))}
              </div>
            ) : (
              <div className='flex w-full flex-col items-center gap-4'>
                <h3 className='text-sm font-bold'>Հաճախորդի կարծիք</h3>
                <div className='w-full rounded-lg bg-muted px-4 py-12 text-center text-muted-foreground'>
                  Կարծիքներ դեռ չկան։ Օգնեք հաճախորդներին ընտրության հարցում,
                  եղեք առաջինը և գրեք Ձեր կարծիքը այս զարդի մասին
                </div>
              </div>
            )}

            <AddReviewDialog haysell_id={haySellId} />
          </div>
          {productData?.related_products &&
            productData.related_products.length > 0 && (
              <CardCarousel
                title='Նմանատիպ զարդեր'
                href='/'
                showDots={false}
                products={productData.related_products}
              />
            )}
        </>
      )}
    </div>
  )
}

export default Product
