'use client'

import Image from 'next/image'
import Link from 'next/link'

import { useMutation, useQueryClient } from '@tanstack/react-query'
import { AxiosError } from 'axios'
import Cookies from 'js-cookie'
import { Heart } from 'lucide-react'
import { useRouter } from 'next-nprogress-bar'

import { Button, buttonVariants } from '@/components/ui/button'
import {
  Dialog,
  DialogContent,
  DialogHeader,
  DialogTrigger,
} from '@/components/ui/dialog'
import { useToast } from '@/components/ui/use-toast'
import { convertToMoney } from '@/lib/utils'
import axiosClient from '@/services/axiosClient'
import { useUserStore } from '@/store/user'

const AddToFavoritesDialog = ({
  isInProduct,
  haysellId,
  isFavoriteInitial,
  image,
}: {
  isInProduct?: boolean
  haysellId: number
  isFavoriteInitial: boolean
  image: string
}) => {
  const token = Cookies.get('siamoods_token')
  const router = useRouter()
  const { currentFavorites, addOrRemoveFavorite } = useUserStore()
  const favoriteProduct = currentFavorites?.find(
    (item) => item.haysell_id === haysellId
  )
  const isFavorite = favoriteProduct
    ? favoriteProduct.isFavorite
    : isFavoriteInitial
  const queryClient = useQueryClient()
  const { toast } = useToast()

  const mutation = useMutation({
    mutationFn: (id: number) => {
      if (isFavorite) {
        return axiosClient.delete(`api/remove-favorite/${id}`)
      } else {
        return axiosClient.post(`api/add-favorite`, {
          haysell_id: id,
        })
      }
    },
    onSuccess: (_, variables) => {
      const id = variables
      queryClient.invalidateQueries({ queryKey: ['favorites'] })
      addOrRemoveFavorite(id)
      // toast({
      //   title: isFavorite ? 'Ապրանքը հեռացված է' : 'Ապրանքը ավելացված է',
      // })
      // if (isFavorite) {
      //   toast({
      //     title: 'Ապրանքը հեռացված է',
      //   })
      // }
    },
    onError: (error) => {
      if (error instanceof AxiosError) {
        if (error?.response?.data.message) {
          toast({
            title: error.response.data.message,
            variant: 'destructive',
          })
        }
      }
    },
  })

  return (
    <Dialog>
      <DialogTrigger
        asChild
        onClick={(e) => {
          if (isFavorite || !token) e.preventDefault()
          if (!token) {
            router.push('/login')
            return
          }
          mutation.mutate(haysellId)
        }}
      >
        {isInProduct ? (
          <Button variant='outline' className='text-red-500 hover:text-red-500'>
            <Heart fill={isFavorite ? '#ef4444' : 'none'} />
          </Button>
        ) : (
          <Button variant='ghost' size='iconSm'>
            <Heart
              size={19}
              className='text-red-500'
              fill={isFavorite ? '#ef4444' : 'none'}
            />
          </Button>
        )}
      </DialogTrigger>
      <DialogContent className='gap-0 p-0'>
        <DialogHeader className='bg-muted p-4 text-start text-muted-foreground'>
          <h2>Ապրանքն ավելացվել է Ձեր ցանկալի ապրանքների ցանկում</h2>
        </DialogHeader>
        <div className='flex gap-4 p-8 md:p-4 sm:flex-col sm:items-center'>
          <Link href='/'>
            <Image
              src={image}
              alt='product'
              width={80}
              height={80}
              className='aspect-square rounded-md object-cover'
            />
          </Link>
          <div className='flex flex-1 flex-col items-start gap-1 sm:items-center'>
            <Link
              href='/'
              className='line-clamp-1 text-base transition-colors hover:text-primary'
            >
              Արծաթյա ականջօղեր SE214RD4W
            </Link>
            <div>Չափս: 16</div>
          </div>
          <div>2 x {convertToMoney(4770)}</div>
        </div>
        <div className='space-y-4 border-t p-4'>
          <div className='flex justify-between pr-4 sm:flex-col-reverse sm:gap-4'>
            <DialogTrigger asChild className='mt-0'>
              <Button variant='ghost'>Շարունակել գնումները</Button>
            </DialogTrigger>
            <Link href='/favorites' className={buttonVariants()}>
              Դիտել ցանկալի ապրանքների ցանկը
            </Link>
          </div>
        </div>
      </DialogContent>
    </Dialog>
  )
}

export default AddToFavoritesDialog
