'use client'

import { useState } from 'react'

import { useQuery } from '@tanstack/react-query'
import { AxiosError } from 'axios'
import Cookies from 'js-cookie'
import ReCAPTCHA from 'react-google-recaptcha'
import * as z from 'zod'

import { Button } from '@/components/ui/button'
import {
  Dialog,
  DialogContent,
  DialogFooter,
  DialogHeader,
  DialogTrigger,
} from '@/components/ui/dialog'
import { Input } from '@/components/ui/input'
import { Label } from '@/components/ui/label'
import { useToast } from '@/components/ui/use-toast'
import { getAccount } from '@/server/getAccount'
import axiosClient from '@/services/axiosClient'

const NotifyIfAvailableDialog = ({
  haysellId,
  variationId,
}: {
  haysellId: number | undefined
  variationId: string | undefined
}) => {
  const token = Cookies.get('siamoods_token')
  const queryInfo = useQuery({
    queryKey: ['account'],
    queryFn: getAccount,
    enabled: !!token,
  })
  const accountData = queryInfo?.data
  const [open, setOpen] = useState(false)
  const [email, setEmail] = useState('')
  const { toast } = useToast()
  const emailSchema = z.string().email()
  const [loading, setLoading] = useState(false)
  const [captchaToken, setCaptchaToken] = useState('')

  async function handleNotify() {
    try {
      emailSchema.parse(accountData?.email || email)
    } catch (error) {
      toast({
        title: 'Սխալ էլ․ հասցե',
        variant: 'destructive',
      })
      return
    }

    setLoading(true)
    try {
      await axiosClient.post('api/add-waiting', {
        email: accountData?.email || email,
        haysell_id: haysellId,
        variation_id: variationId,
        captchaToken: captchaToken || undefined,
      })
      toast({
        title: 'Ձեզ կտեղեկացնենք առկայության դեպքում',
      })
    } catch (error) {
      if (error instanceof AxiosError) {
        if (error?.response?.data.message) {
          toast({
            title: error.response.data.message,
            variant: 'destructive',
          })
        }
      }
    }
    setOpen(false)
    setLoading(false)
  }

  return (
    <Dialog
      open={open}
      onOpenChange={(isOpen) => {
        if (token) return
        if (accountData?.email && !token) {
          handleNotify()
        }
        setOpen(isOpen)
        if (isOpen) {
          setEmail(accountData?.email || '')
        }
      }}
      modal={false}
    >
      <DialogTrigger asChild>
        <Button
          type='button'
          onClick={() => {
            if (accountData?.email && token) {
              handleNotify()
            }
          }}
        >
          Տեղեկացնել առկայության դեպքում
        </Button>
      </DialogTrigger>
      {open && (
        <div
          className='fixed inset-0 z-20 bg-black/50'
          onClick={() => setOpen(false)}
        />
      )}
      <DialogContent
        className='max-w-xl gap-0 p-0'
        onOpenAutoFocus={(e) => e.preventDefault()}
      >
        <DialogHeader className='bg-muted p-4 text-start text-muted-foreground'>
          <h2>Տեղեկացնել առկայության դեպքում</h2>
        </DialogHeader>
        <div className='flex flex-col items-center gap-4 p-8'>
          <Label htmlFor='email' className='self-start'>
            Էլ․ հասցե
          </Label>
          <Input
            id='email'
            value={email}
            onChange={(e) => setEmail(e.target.value)}
            onKeyDown={(e) => {
              if (e.key === 'Enter' && email) {
                if (loading) return
                handleNotify()
              }
            }}
            type='email'
          />
          <ReCAPTCHA
            sitekey={process.env.NEXT_PUBLIC_RE_CAPTCHA_KEY as string}
            onChange={(token) => {
              if (token) {
                setCaptchaToken(token)
              }
            }}
          />
        </div>
        <DialogFooter className='items-end border-t p-4'>
          <Button type='button' disabled={loading} onClick={handleNotify}>
            Հաստատել
          </Button>
        </DialogFooter>
      </DialogContent>
    </Dialog>
  )
}

export default NotifyIfAvailableDialog
