'use client'

import Link from 'next/link'
import { usePathname } from 'next/navigation'
import Script from 'next/script'
import { Fragment } from 'react'

import {
  Breadcrumb,
  BreadcrumbItem,
  BreadcrumbLink,
  BreadcrumbList,
  BreadcrumbSeparator,
} from '@/components/ui/breadcrumb'
import { useUserStore } from '@/store/user'

const breadcrumbNavigationArmenianNames: { [key: string]: string } = {
  blog: 'Բլոգ',
  store: 'Օնլայն խանութ',
  product: 'Օնլայն խանութ',
  akandjogher: 'Ականջօղեր',
  mekhik: 'Մեխիկ',
  qaf: 'Քաֆ',
  oghak: 'Օղակ',
  magltcogh: 'Մագլցող',
  shghthayov: 'Շղթայով',
  kisaoghak: 'Կիսաողակ',
  aparandjanner: 'Ապարանջաններ',
  kakhazarder: 'Կախազարդեր',
  mataniner: 'Մատանիներ',
  vznotcner: 'Վզնոցներ',
  shghthaner: 'Շղթաներ',
  krtsqazarder: 'Կրծքազարդեր',
  cart: 'Զամբյուղ',
  api: 'API',
  'gift-card': 'Նվեր Քարտ',
  'digital-gift-card': 'Օնլայն Նվեր Քարտ',
  profile: 'Հաշիվ',
  orders: 'Պատվերներ',
  favorites: 'Նախընտրածներ',
  'change-password': 'Փոխել Գաղտնաբառը',
  checkout: 'Վճարում',
  'checkout-haypost': 'Վճարում Հայփոստ',
}

const BreadcrumbNavigation = () => {
  const pathname = usePathname()
  const hideBreadCrumb = ['/', '/login', '/register', '/forgot-password']
  const hideBreadCrumbStartsWith = [
    '/reset-password',
    '/unsubscribe',
    '/guest-orders',
    '/api',
  ]
  const currentBreadcrumbItem = useUserStore(
    (state) => state.currentBreadcrumbItem
  )

  const shouldHideBreadcrumb =
    hideBreadCrumb.includes(pathname) ||
    hideBreadCrumbStartsWith.some((path) => pathname.startsWith(path))

  // Generate breadcrumb items and JSON-LD data
  const generateBreadcrumbData = () => {
    const items = []
    const pathSegments = pathname.split('/')

    items.push({
      position: 1,
      name: 'Գլխավոր',
      item: process.env.NEXT_PUBLIC_APP_URL,
    })

    pathSegments.forEach((path, index) => {
      if (!path) return

      let url = pathSegments.slice(0, index + 1).join('/')
      if (pathname.includes(currentBreadcrumbItem.slug) && path === 'product') {
        url = url.replace('product', 'store')
      }

      const name =
        path === currentBreadcrumbItem.slug
          ? currentBreadcrumbItem.name
          : breadcrumbNavigationArmenianNames[path] ||
            path.charAt(0).toUpperCase() + path.slice(1)

      items.push({
        position: items.length + 1,
        name,
        item: `${process.env.NEXT_PUBLIC_APP_URL}${url}`,
      })
    })

    return items
  }

  if (shouldHideBreadcrumb) {
    return null
  }

  const breadcrumbData = generateBreadcrumbData()
  const jsonLd = {
    '@context': 'https://schema.org',
    '@type': 'BreadcrumbList',
    itemListElement: breadcrumbData.map((item) => ({
      '@type': 'ListItem',
      position: item.position,
      name: item.name,
      item: item.item,
    })),
  }

  return (
    <div>
      <Script id='breadcrumb-jsonld' type='application/ld+json'>
        {JSON.stringify(jsonLd)}
      </Script>
      <div className='container py-6'>
        <Breadcrumb>
          <BreadcrumbList>
            <BreadcrumbItem>
              <BreadcrumbLink asChild>
                <Link href='/'>Գլխավոր</Link>
              </BreadcrumbLink>
            </BreadcrumbItem>
            {pathname !== '/' && <BreadcrumbSeparator />}
            {pathname.split('/').map((path, index, array) => {
              if (!path) return null
              let url = array.slice(0, index + 1).join('/')
              if (
                pathname.includes(currentBreadcrumbItem.slug) &&
                path === 'product'
              ) {
                url = url.replace('product', 'store')
              }
              const name =
                path === currentBreadcrumbItem.slug
                  ? currentBreadcrumbItem.name
                  : breadcrumbNavigationArmenianNames[path] ||
                    path.charAt(0).toUpperCase() + path.slice(1)

              return (
                <Fragment key={index}>
                  <BreadcrumbItem>
                    <BreadcrumbLink asChild>
                      <Link href={url}>{name}</Link>
                    </BreadcrumbLink>
                  </BreadcrumbItem>
                  {index !== array.length - 1 && <BreadcrumbSeparator />}
                </Fragment>
              )
            })}
          </BreadcrumbList>
        </Breadcrumb>
      </div>
    </div>
  )
}

export default BreadcrumbNavigation
