'use client'

import {
  Accordion,
  AccordionContent,
  AccordionItem,
  AccordionTrigger,
} from '@/components/ui/accordion'
import { Button, buttonVariants } from '@/components/ui/button'
import { Sheet, SheetContent, SheetTrigger } from '@/components/ui/sheet'
import { cn, countDashes } from '@/lib/utils'
import { Menu } from 'lucide-react'
import Link from 'next/link'
import { useState } from 'react'
import GlobalSearch from './GlobalSearch'
import Cookies from 'js-cookie'
import type { HeaderData } from '@/types'

type NavItem = {
  name: string
  href: string
  subItems?: NavItem[]
}

const AccordionComponent = ({
  item,
  indexPath,
  values,
  setValues,
}: {
  item: NavItem
  indexPath: string
  values: string[]
  setValues: React.Dispatch<React.SetStateAction<string[]>>
}) => {
  const value = `item-${indexPath}`

  const handleClick = (e: React.MouseEvent) => {
    e.stopPropagation()

    if (item.subItems) {
      setValues((prev) => {
        if (prev.includes(value)) {
          return prev.filter((v) => !v.startsWith(value))
        } else {
          if (countDashes(value) === 1) {
            return [value]
          } else {
            return [...prev, value]
          }
        }
      })
    }
  }

  return (
    <AccordionItem
      key={indexPath}
      value={value}
      className='py-4 pl-4'
      onClick={handleClick}
    >
      <SheetTrigger asChild>
        <Link href={item.href}>{item.name}</Link>
      </SheetTrigger>
      {item.subItems && item.subItems.length > 0 && (
        <AccordionTrigger
          className='h-0 justify-end p-0 text-muted-foreground'
          iconClassName='size-8 -translate-y-2'
        ></AccordionTrigger>
      )}
      <AccordionContent>
        {item.subItems?.map((subItem, subIndex) => (
          <AccordionComponent
            item={subItem}
            indexPath={`${indexPath}-${subIndex}`}
            key={subIndex}
            values={values}
            setValues={setValues}
          />
        ))}
      </AccordionContent>
    </AccordionItem>
  )
}

const MobileNav = ({ headerData }: { headerData: HeaderData[] }) => {
  const [values, setValues] = useState<string[]>([])
  const token = Cookies.get('siamoods_token')

  return (
    <div className='hidden md:block'>
      <Sheet onOpenChange={() => setValues([])}>
        <SheetTrigger asChild>
          <Button variant='outline' className='border-none' size='icon'>
            <Menu />
          </Button>
        </SheetTrigger>
        <SheetContent
          className='pt-12'
          onOpenAutoFocus={(e) => e.preventDefault()}
        >
          <GlobalSearch />
          <div className=' max-h-[60vh] overflow-y-auto px-2'>
            <Accordion type='multiple' className='w-full' value={values}>
              {headerData.map((item, index) => (
                <AccordionComponent
                  item={item}
                  indexPath={index.toString()}
                  key={index}
                  values={values}
                  setValues={setValues}
                />
              ))}
            </Accordion>
          </div>
          <div className='flex flex-col gap-4 pt-8'>
            <SheetTrigger asChild>
              <Link
                href={token ? '/profile' : '/login'}
                className={cn(buttonVariants())}
              >
                {token ? 'Հաշիվ' : 'Մուտք'}
              </Link>
            </SheetTrigger>
            <SheetTrigger asChild>
              <Link
                href={token ? '/favorites' : '/login'}
                className={cn(
                  buttonVariants({
                    variant: 'outlineSecondary',
                  })
                )}
              >
                Նախըտրածներ
              </Link>
            </SheetTrigger>
          </div>
        </SheetContent>
      </Sheet>
    </div>
  )
}

export default MobileNav
